/** @file
 *
 * (c) Copyright 2007 by Intra2net AG
 * 
 * info@intra2net.com
 */

#include "async_pipe.hpp"

#include <functional>
#include <boost/bind.hpp>

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>


namespace AsyncIo
{


/*
 * Implementation of SimplePipe
 */

SimplePipe::SimplePipe()
{
   m_signal_read.connect(boost::bind(&SimplePipe::slotReceived,this));
} // eo SimplePipe::SimplePipe()


SimplePipe::~SimplePipe()
{
} // eo SimplePipe::~SimplePipe()


/**
 * makes a pipe.
 * This method connects itself with a newly created peer object with a bidirectional pipe.
 * @return the peer pipe object.
 */
bool SimplePipe::makePipe(SimplePipe& peer)
{
   close(); // just in case...

   int fds[2];

   int res= ::socketpair( AF_UNIX, SOCK_STREAM, 0, fds);

   if (res)
   {
      m_errno= errno;
      return false;
   }
   else
   {
      m_errno= 0;
   }

   peer.close(); // just in case

   setWriteFd(fds[0]);
   setReadFd(fds[0]);

   peer.setWriteFd(fds[1]);
   peer.setReadFd(fds[1]);

   return true;
} // eo SimplePipe.:makePipe()


/**
 * sends a string through the pipe.
 * @param data the data which should be sent to the other side.
 */
void SimplePipe::sendString(const std::string& data)
{
   lowSend(data);
} // eo SimplePipe::sendString(const std::string&)


/**
 * emits the signal signalReceived with the received data.
 * This slot is connected to IOImplementation::m_signal_read.
 */
void SimplePipe::slotReceived()
{
   std::string data;
   data.swap(m_input_buffer);
   signal_received_string(data);
} // eo SimplePipe::slotReceived()


} // eo namespace AsyncIo
