/** @file
 *
 * provides timer objects based on the TimerBase class 
 *
 * (c) Copyright 2007 by Intra2net AG
 * 
 * info@intra2net.com
 */

#ifndef __ASYNC_TIMER_HPP__
#define __ASYNC_TIMER_HPP__

#include "async_io.hpp"

#include <boost/signal.hpp>

namespace AsyncIo
{


class SimpleTimer : public TimerBase
{
   public:

      typedef boost::signal<void()>       TimerSignal;
      typedef boost::signal<void(SimpleTimer*)> TimerSignalP;

   public:

      SimpleTimer();

      // convenience constructors for simple timeouts:
      SimpleTimer(const MilliTime& delta, const TimerSignal::slot_function_type& action);
      SimpleTimer(long milli_seonds_delta, const TimerSignal::slot_function_type& action);

      // add actions:
      void addAction( const TimerSignal::slot_function_type& action );
      void addActionP( const TimerSignalP::slot_function_type& action );

      // start the timer:
      void startTimer( const MilliTime& delta );
      void startTimerMS( long milli_seconds );

      // stop the timer:
      void stopTimer();

   protected:
      MilliTime    m_delta;

      TimerSignal  m_timer_signal;
      TimerSignalP m_timer_signal_p;

      virtual void execute();
}; // eo class SimpleTimer


} // eo namespace AsyncIo

#endif
