/** 
 * @file
 * @brief
 *
 * @author Reinhard Pfau \<reinhard.pfau@intra2net.com\>
 *
 * @copyright &copy; Copyright 2008 Intra2Net AG
 * @license LPGL
 * @contact info@intra2net.com
 *
 */

#ifndef __ASYNCIO__TIME_TOOLS_HPP__
#define __ASYNCIO__TIME_TOOLS_HPP__

namespace AsyncIo
{
namespace Utils
{

/**
 * @brief structure for storing (a point in time as) seconds and milliseconds.
 */
struct MilliTime
{
    long mt_sec;
    long mt_msec;

    MilliTime(long sec=0, long msec=0);

    void set(long sec, long msec=0);

    inline long long get_milliseconds() const
    {
        return ((long long)mt_sec * 1000L + mt_msec);
    } // eo get_milliseconds

    void normalize();

    bool operator < (MilliTime& other);
    bool operator == (MilliTime& other);

    MilliTime& operator -= (const MilliTime& lhs);
    MilliTime& operator += (const MilliTime& lhs);

}; // eo struct MilliTime


inline MilliTime operator + (const MilliTime& rhs, const MilliTime& lhs)
{
    MilliTime t(rhs);
    return t+= lhs;
} // eo operator + (const MilliTime& rhs, const MilliTime lhs)

inline MilliTime operator - (const MilliTime& rhs, const MilliTime& lhs)
{
    MilliTime t(rhs);
    return t-= lhs;
} // eo operator - (const MilliTime& rhs, const MilliTime lhs)


inline bool operator <= (MilliTime& rhs, MilliTime& lhs)
{
    return (rhs<lhs) || (rhs==lhs);
} // eo operator <= (MilliTime& rhs, MilliTime& lhs)



/*
** tool functions:
*/


void get_current_real_time(MilliTime& mt);
void get_current_monotonic_time(MilliTime& mt);


} // eo namespace Utils
} // eo namespace AsyncIo



#endif